//
//  HomeHotDestinationCell.m
//  Ykly
//
//  Created by 郭超 on 2019/7/19.
//  Copyright © 2019 YingKeTravel co.,Ltd. All rights reserved.
//

#import "HomeHotDestinationCell.h"
@interface HomeHotDestinationCell ()
/**
 * 背景图
 */
@property (nonatomic, strong) UIImageView * backgroundImg;
/**
 * 出发地
 */
@property (nonatomic, strong) YYLabel * cityLab;
/**
 * 标题
 */
@property (nonatomic, strong) UILabel * titleLab;
/**
 * 类型
 */
@property (nonatomic, strong) UILabel * typeLab;
/**
 * 价格
 */
@property (nonatomic, strong) UILabel * moneyLab;

@end

@implementation HomeHotDestinationCell
- (UICollectionViewLayoutAttributes*)preferredLayoutAttributesFittingAttributes:(UICollectionViewLayoutAttributes*)layoutAttributes {
    [self setNeedsLayout];
    [self layoutIfNeeded];
    CGSize size = [self.contentView systemLayoutSizeFittingSize: layoutAttributes.size];
    CGRect cellFrame = layoutAttributes.frame;
    cellFrame.size.height= size.height;
    layoutAttributes.frame= cellFrame;
    [self.superview setNeedsLayout];
    [self.superview layoutIfNeeded];
    return layoutAttributes;
}

- (instancetype)initWithFrame:(CGRect)frame {
    
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = WHITE_COLOR;
        [self setUpSubViewsWithConstraints];
    }
    return self;
}

- (void)layoutSubviews {
    
    [super layoutSubviews];
    
//    [YkTool maskLayerView:self.contentView byRoundingCorners:UIRectCornerAllCorners maskcornerRadii:CGSizeMake(10, 10)];
//    [YkTool maskLayerView:self.cityLab byRoundingCorners:UIRectCornerBottomRight maskcornerRadii:CGSizeMake(10, 10)];
    self.layer.cornerRadius = 10;
    self.layer.shadowOffset = CGSizeMake(0,0);
    self.layer.shadowOpacity = 1;
    self.layer.shadowColor = [UIColor colorWithHexString:@"#DEDEDE"].CGColor;
}

- (void)setUpSubViewsWithConstraints {
    
    [self.contentView addSubview:self.backgroundImg];
    [self.backgroundImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.contentView.mas_top);
        make.left.mas_equalTo(self.contentView.mas_left);
        make.width.mas_equalTo(adaptX(170));
        make.height.mas_equalTo(adaptX(170));
        make.right.mas_equalTo(self.contentView.mas_right).priorityMedium();
    }];
    
    [self.backgroundImg addSubview:self.cityLab];
    [self.cityLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.backgroundImg.mas_left);
        make.top.mas_equalTo(self.backgroundImg.mas_top);
        make.height.mas_equalTo(adaptX(23));
    }];
    
    [self.contentView addSubview:self.titleLab];
    [self.titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.backgroundImg.mas_bottom).mas_offset(12);
        make.left.mas_equalTo(self.contentView.mas_left).mas_offset(10);
        make.width.mas_lessThanOrEqualTo(adaptX(150));
//        make.height.mas_lessThanOrEqualTo(45);
    }];
    [self.contentView addSubview:self.moneyLab];
    [self.moneyLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.titleLab.mas_bottom).offset(adaptX(7));
        make.right.mas_equalTo(self.contentView.mas_right).offset(-10);
//        make.left.mas_greaterThanOrEqualTo(self.contentView.mas_left);
        make.height.mas_equalTo(adaptX(16));
        make.bottom.mas_equalTo(self.contentView.mas_bottom).offset(-adaptX(12));
    }];
    
    [self.contentView addSubview:self.typeLab];
    [self.typeLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(self.moneyLab.mas_bottom);
        make.left.mas_equalTo(self.titleLab.mas_left);
        make.height.mas_equalTo(13);
    }];
    
}

- (void)setModel:(HomeHotDestinationModel *)model {
    _model = model;
    
//    [self.backgroundImg sd_setImageWithURL:[NSURL URLWithString:model.product_img] placeholderImage:PLACEHOLDER_IMAGE];
    [self.backgroundImg setImageWithURL:[NSURL URLWithString:model.product_img] placeholder:PLACEHOLDER_IMAGE];
    self.cityLab.text = model.departure_city;
    self.typeLab.text = model.product_type_name;
    self.titleLab.text = model.product_name;
    NSString * moneyNum = [NSString stringWithFormat:@"%@",[NSNumber numberWithString:model.product_price].stringValue];
    
    NSString * moneyContent = [NSString stringWithFormat:@"¥ %@ 起", moneyNum];
    NSMutableAttributedString * moneyText = [[NSMutableAttributedString alloc] initWithString:moneyContent];
    moneyText.font = [UIFont fontWithName:@"PingFangSC-Regular" size:10];
    moneyText.color = [UIColor colorWithHexString:PriceColorHex];
    
    NSRange numRange = [moneyContent rangeOfString:moneyNum];
    [moneyText setFont:[UIFont fontWithName:@"PingFangSC-Regular" size:16] range:numRange];
    
    self.moneyLab.attributedText = moneyText;
    self.moneyLab.textAlignment = NSTextAlignmentRight;
}
- (UIImageView *)backgroundImg
{
    if (!_backgroundImg) {
        _backgroundImg = [UIImageView new];
        _backgroundImg.contentMode = UIViewContentModeScaleAspectFill;
        _backgroundImg.layer.masksToBounds= YES;
    }return _backgroundImg;
}

- (YYLabel *)cityLab
{
    if (!_cityLab) {
        _cityLab = [YYLabel new];
        _cityLab.backgroundColor = [UIColor colorWithHexString:@"000000"];
        _cityLab.textColor = [UIColor whiteColor];
        _cityLab.font =  kPingFangRegular(12);
        _cityLab.textContainerInset = UIEdgeInsetsMake(0, adaptX(25), 0, adaptX(25));
        _cityLab.textAlignment = NSTextAlignmentLeft;
        [_cityLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisHorizontal];
    }return _cityLab;
}

- (UILabel *)titleLab
{
    if (!_titleLab) {
        _titleLab = [UILabel new];
        _titleLab.numberOfLines = 2;
        _titleLab.font =  kPingFangRegular(15);
        _titleLab.textColor = UIColorHex(000000);
        _titleLab.textAlignment = NSTextAlignmentLeft;
        _titleLab.preferredMaxLayoutWidth = adaptX(150);
    }return _titleLab;
}

- (UILabel *)typeLab
{
    if (!_typeLab) {
        _typeLab = [UILabel new];
        _typeLab.font =  kPingFangRegular(12);
        _typeLab.textColor = [UIColor colorWithHexString:@"#666666"];
        _typeLab.textAlignment = NSTextAlignmentLeft;
    }return _typeLab;
}

- (UILabel *)moneyLab
{
    if (!_moneyLab) {
        _moneyLab = [UILabel new];
        _moneyLab.textAlignment = NSTextAlignmentRight;
    }return _moneyLab;
}
@end
