//
//  HomeSelectHotelCell.m
//  Ykly
//
//  Created by 郭超 on 2019/7/19.
//  Copyright © 2019 YingKeTravel co.,Ltd. All rights reserved.
//

#import "HomeSelectHotelCell.h"
@interface HomeSelectHotelCell ()
/**
 * 背景图
 */
@property (nonatomic, strong) UIImageView * backgroundImg;
/**
 * 出发地
 */
@property (nonatomic, strong) YYLabel * cityLab;
/**
 * 标题
 */
@property (nonatomic, strong) YYLabel * titleLab;
/**
 * 类型
 */
@property (nonatomic, strong) UILabel * typeLab;
/**
 * 价格
 */
@property (nonatomic, strong) YYLabel * moneyLab;
@end

@implementation HomeSelectHotelCell
- (instancetype)initWithFrame:(CGRect)frame {
    
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = WHITE_COLOR;
        [self setUpSubViewsWithConstraints];
    }
    return self;
}

- (void)layoutSubviews {
    
    [super layoutSubviews];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.mas_equalTo(self);
    }];
    
    [self.backgroundImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.mas_equalTo(self.contentView);
        make.width.mas_equalTo((kScreenWidth - 3 * 12)/2);
        make.height.mas_equalTo((kScreenWidth - 3 * 12)/2);
    }];
    
    [self.cityLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.mas_equalTo(0);
    }];
    
    self.titleLab.preferredMaxLayoutWidth = self.width - 20;
    [self.titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.backgroundImg.mas_bottom).mas_offset(12);
        make.left.mas_equalTo(self.contentView.mas_left).mas_offset(10);
        make.right.mas_equalTo(self.contentView.mas_right).mas_offset(-10);
        make.height.mas_lessThanOrEqualTo(45);
    }];
    
    [self.typeLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(self.moneyLab.mas_bottom);
        make.left.mas_equalTo(self.titleLab.mas_left);
        make.height.mas_equalTo(13);
        make.width.mas_greaterThanOrEqualTo(0);
    }];
    
    [self.moneyLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.titleLab.mas_bottom).mas_offset(8);
        make.left.mas_equalTo(self.typeLab.mas_right).mas_offset(8);
        make.right.mas_equalTo(self.titleLab.mas_right);
        make.height.mas_equalTo(20);
    }];
    
    [self.contentView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(self.typeLab.mas_bottom).offset(12);
    }];
    
//    [YkTool maskLayerView:self.contentView byRoundingCorners:UIRectCornerAllCorners maskcornerRadii:CGSizeMake(10, 10)];
//    [YkTool maskLayerView:self.cityLab byRoundingCorners:UIRectCornerBottomRight maskcornerRadii:CGSizeMake(10, 10)];
    self.layer.cornerRadius = 10;
    self.layer.shadowOffset = CGSizeMake(0,0);
    self.layer.shadowOpacity = 1;
    self.layer.shadowColor = [UIColor colorWithHexString:@"#DEDEDE"].CGColor;
}

- (void)setUpSubViewsWithConstraints {
    [self.contentView addSubview:self.backgroundImg];
    [self.backgroundImg addSubview:self.cityLab];
    [self.contentView addSubview:self.titleLab];
    [self.contentView addSubview:self.moneyLab];
    [self.contentView addSubview:self.typeLab];
}

- (UICollectionViewLayoutAttributes*)preferredLayoutAttributesFittingAttributes:(UICollectionViewLayoutAttributes*)layoutAttributes {
    [self setNeedsLayout];
    [self layoutIfNeeded];
    CGSize size = [self.contentView systemLayoutSizeFittingSize:layoutAttributes.size];
    CGRect cellFrame = layoutAttributes.frame;
    cellFrame.size.height= size.height;
    layoutAttributes.frame= cellFrame;
    return layoutAttributes;
}

- (void)setModel:(HomeSelectHotelModel *)model {
    _model = model;
    
//    [self.backgroundImg sd_setImageWithURL:[NSURL URLWithString:model.product_img] placeholderImage:PLACEHOLDER_IMAGE];
    [self.backgroundImg setImageWithURL:[NSURL URLWithString:model.product_img] placeholder:PLACEHOLDER_IMAGE];
    self.backgroundImg.contentMode = UIViewContentModeScaleAspectFill;
    self.backgroundImg.layer.masksToBounds= YES;
    self.cityLab.text = model.departure_city;
    self.titleLab.text = model.hotel_name;
    self.typeLab.text = model.hotel_level_name;
    
    NSString * moneyNum = [NSString stringWithFormat:@"%@",[NSNumber numberWithString:model.hotel_price].stringValue];
    NSString * rmb = @"¥";
    NSString * qi = @"起";
    NSString * moneyContent = [NSString stringWithFormat:@"%@ %@ %@",rmb,moneyNum,qi];
    NSMutableAttributedString * moneyText = [[NSMutableAttributedString alloc] initWithString:moneyContent];
    moneyText.font = [UIFont fontWithName:@"PingFangSC-Regular" size:10];
    moneyText.color = [UIColor colorWithHexString:PriceColorHex];
    
    NSRange numRange = [moneyContent rangeOfString:moneyNum];
    [moneyText setFont:[UIFont fontWithName:@"PingFangSC-Regular" size:16] range:numRange];
    
    self.moneyLab.attributedText = moneyText;
    self.moneyLab.textAlignment = NSTextAlignmentRight;
}

- (UIImageView *)backgroundImg
{
    if (!_backgroundImg) {
        _backgroundImg = [UIImageView new];
    }return _backgroundImg;
}

- (YYLabel *)cityLab
{
    if (!_cityLab) {
        _cityLab = [YYLabel new];
        _cityLab.textContainerInset = UIEdgeInsetsMake(0, 13, 0, 13);
        _cityLab.font =  kPingFangRegular(12);
        _cityLab.backgroundColor = [UIColor colorWithHexString:@"#000000"];
        _cityLab.textColor = [UIColor whiteColor];
        _cityLab.textAlignment = NSTextAlignmentLeft;
        [_cityLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisHorizontal];
    }return _cityLab;
}

- (YYLabel *)titleLab
{
    if (!_titleLab) {
        _titleLab = [YYLabel new];
        _titleLab.numberOfLines = 2;
        _titleLab.font =  kPingFangRegular(15);
        _titleLab.textColor = [UIColor colorWithHexString:@"000000"];
        _titleLab.textAlignment = NSTextAlignmentLeft;
        [_titleLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisHorizontal];
    }return _titleLab;
}

- (UILabel *)typeLab
{
    if (!_typeLab) {
        _typeLab = [UILabel new];
        _typeLab.font =  kPingFangRegular(12);
        _typeLab.textColor = [UIColor colorWithHexString:@"#666666"];
        _typeLab.textAlignment = NSTextAlignmentLeft;
        [_typeLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisHorizontal];
    }return _typeLab;
}

- (YYLabel *)moneyLab
{
    if (!_moneyLab) {
        _moneyLab = [YYLabel new];
        _moneyLab.textContainerInset = UIEdgeInsetsMake(0, 0, 0, 0);
        [_moneyLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisHorizontal];
    }return _moneyLab;
}
@end
