//
//  HomeSelectOptimalCell.m
//  Ykly
//
//  Created by 郭超 on 2019/7/19.
//  Copyright © 2019 YingKeTravel co.,Ltd. All rights reserved.
//

#import "HomeSelectOptimalCell.h"
@interface HomeSelectOptimalCell ()
/**
 * 背景图
 */
@property (nonatomic, strong) UIImageView * backgroundImg;
/**
 * 标题
 */
@property (nonatomic, strong) YYLabel * titleLab;
/**
 * 价格
 */
@property (nonatomic, strong) YYLabel * moneyLab;
@end

@implementation HomeSelectOptimalCell
- (instancetype)initWithFrame:(CGRect)frame {
    
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = WHITE_COLOR;
        [self setUpSubViewsWithConstraints];
    }
    return self;
}

- (void)layoutSubviews {
    
    [super layoutSubviews];
    
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.mas_equalTo(self);
    }];
    [self.backgroundImg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.right.mas_equalTo(self.contentView);
        make.height.mas_equalTo(self.width);
    }];
    self.titleLab.preferredMaxLayoutWidth = self.width - 20;//要是设置多行Label的话,必须设置此属性要是设置多行Label的话,必须设置此属性要是设置多行Label的话,必须设置此属性要是设置多行Label的话,必须设置此属性
    [self.titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.backgroundImg.mas_bottom).mas_offset(12);
        make.left.mas_equalTo(self.contentView.mas_left).mas_offset(10);
        make.right.mas_equalTo(self.contentView.mas_right).mas_offset(-10);
        make.height.mas_lessThanOrEqualTo(45);
    }];
    [self.moneyLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.titleLab.mas_bottom).mas_offset(8);
        make.left.mas_equalTo(self.titleLab.mas_left);
        make.right.mas_equalTo(self.titleLab.mas_right);
        make.height.mas_equalTo(20);
    }];
    
    [self.contentView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(self.moneyLab.mas_bottom).offset(12);
    }];

    
//    [YkTool maskLayerView:self.contentView byRoundingCorners:UIRectCornerAllCorners maskcornerRadii:CGSizeMake(10, 10)];
    self.layer.cornerRadius = 10;
    self.layer.shadowOffset = CGSizeMake(0,0);
    self.layer.shadowOpacity = 1;
    self.layer.shadowColor = [UIColor colorWithHexString:@"#DEDEDE"].CGColor;
}

- (void)setUpSubViewsWithConstraints {
    
    
    [self.contentView addSubview:self.backgroundImg];
    
    [self.contentView addSubview:self.titleLab];
    
    [self.contentView addSubview:self.moneyLab];
}

- (void)setModel:(HomeSelectOptimalModel *)model {
    _model = model;
    
//    [self.backgroundImg sd_setImageWithURL:[NSURL URLWithString:model.product_img] placeholderImage:PLACEHOLDER_IMAGE];
    [self.backgroundImg setImageWithURL:[NSURL URLWithString:model.product_img] placeholder:PLACEHOLDER_IMAGE];
    self.backgroundImg.contentMode = UIViewContentModeScaleAspectFill;
    self.backgroundImg.layer.masksToBounds= YES;
    self.titleLab.text = model.product_name;
    
    CGFloat priceFloat = [model.product_price floatValue];
    NSString *price = [NSString stringWithFormat:@"%.2f", priceFloat].copy;
    
    NSString * moneyNum = [self removeSuffix:price];
//    [NSString stringWithFormat:@"%@",[NSNumber numberWithString:model.product_price].stringValue];
    NSString * rmb = @"¥";
    NSString * qi = @"起";
    NSString * moneyContent = [NSString stringWithFormat:@"%@ %@ %@",rmb,moneyNum,qi];
    NSMutableAttributedString * moneyText = [[NSMutableAttributedString alloc] initWithString:moneyContent];
    moneyText.font = [UIFont fontWithName:@"PingFangSC-Regular" size:10];
    moneyText.color = [UIColor colorWithHexString:PriceColorHex];
    
    NSRange numRange = [moneyContent rangeOfString:moneyNum];
    [moneyText setFont:[UIFont fontWithName:@"PingFangSC-Regular" size:16] range:numRange];
    
    self.moneyLab.attributedText = moneyText;
    self.moneyLab.textAlignment = NSTextAlignmentLeft;
}

- (UICollectionViewLayoutAttributes*)preferredLayoutAttributesFittingAttributes:(UICollectionViewLayoutAttributes*)layoutAttributes {
    [self setNeedsLayout];
    [self layoutIfNeeded];
    CGSize size = [self.contentView systemLayoutSizeFittingSize:layoutAttributes.size];
    CGRect cellFrame = layoutAttributes.frame;
    cellFrame.size.height= size.height;
    layoutAttributes.frame= cellFrame;
    return layoutAttributes;
}

- (NSString *)removeSuffix:(NSString *)numberStr{
    if (numberStr.length > 1) {
        
        if ([numberStr componentsSeparatedByString:@"."].count == 2) {
            NSString *last = [numberStr componentsSeparatedByString:@"."].lastObject;
            if ([last isEqualToString:@"00"]) {
                numberStr = [numberStr substringToIndex:numberStr.length - (last.length + 1)];
                return numberStr;
            }else{
                if ([[last substringFromIndex:last.length -1] isEqualToString:@"0"]) {
                    numberStr = [numberStr substringToIndex:numberStr.length - 1];
                    return numberStr;
                }
            }
        }
        return numberStr;
    }else{
        return nil;
    }
}

- (UIImageView *)backgroundImg
{
    if (!_backgroundImg) {
        _backgroundImg = [UIImageView new];
    }return _backgroundImg;
}

- (YYLabel *)titleLab
{
    if (!_titleLab) {
        _titleLab = [YYLabel new];
        _titleLab.numberOfLines = 2;
        _titleLab.font =  kPingFangRegular(15);
        _titleLab.textColor = [UIColor colorWithHexString:@"000000"];
        _titleLab.textAlignment = NSTextAlignmentLeft;
        [_titleLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisVertical];
    }return _titleLab;
}
- (YYLabel *)moneyLab
{
    if (!_moneyLab) {
        _moneyLab = [YYLabel new];
        _moneyLab.textContainerInset = UIEdgeInsetsMake(0, 0, 0, 0);
        [_moneyLab setContentHuggingPriority:UILayoutPriorityRequired forAxis:UILayoutConstraintAxisVertical];
    }return _moneyLab;
}
@end
